<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php
class Paginas extends Controller{
	// Variable protegida
	protected $_template;
	
	// Constructor
	public function __construct()
	{
		parent::Controller();

		// Comprobamos si el usuario logueado es administrador
		$this->access_library->check_access();

		// Cargamos modulos, librerias y demás ficheros necesarios
		$this->load->module_model('admin', 'paginas_model', 'paginas');
		$this->load->module_model('admin', 'navegacion_model', 'navegacion');
	}

	// Métodos públicos
	public function index()
	{
		$data['paginas'] = $this->paginas->listar_paginas();
		
		$this->_template['pagina'] = 'pages/listar';

		$this->system_library->load($this->_template['pagina'], $data, TRUE);
	}

	public function crear()
	{
		$this->form_validation->set_rules('titulo', 'Titulo', 'required|max_length[200]|xss_clean');
		$this->form_validation->set_rules('contenido', 'Contenido', 'required|prep_for_form');
		$this->form_validation->set_rules('estado', 'Estado', 'required|xss_clean');

		$this->form_validation->set_error_delimiters('', '<br />');		
			
		if ($this->form_validation->run() == TRUE)
		{
			$this->paginas->crear_pagina();
			
			if ($this->input->post('add_to_navigation'))
			{
				$url = 'paginas/' . url_title($this->input->post('titulo')) . '/';
				
				$this->navegacion->crear_navegacion_item($this->input->post('titulo'), $this->input->post('titulo'), $url);
			}
			
			$this->session->set_flashdata('message', 'Página creada satisfactoriamente');

			redirect('admin/paginas', 'refresh');
		}
			
		$this->_template['pagina'] = 'pages/crear';
			
		$this->system_library->load($this->_template['pagina'], NULL, TRUE);
	}

	public function editar($id = NULL)
	{
		if ($id == NULL)
		{
			$id = $this->input->post('id');
		}
			
		$this->form_validation->set_rules('titulo', 'Titulo', 'required|max_length[200]|xss_clean');
		$this->form_validation->set_rules('contenido', 'Contenido', 'required|prep_for_form');
		$this->form_validation->set_rules('estado', 'Estado', 'required|xss_clean');

		$this->form_validation->set_error_delimiters('', '<br />');
			
		$data['page_data'] = $this->paginas->listar_pagina($id);
			
		if ($this->form_validation->run() == TRUE)
		{
			$this->paginas->editar_pagina($id);
			$this->session->set_flashdata('message', 'Página editada satisfactoriamente');

			redirect('admin/paginas', 'refresh');
		}
		
		$this->_template['pagina'] = 'pages/editar';
			
		$this->system_library->load($this->_template['pagina'], $data, TRUE);
	}

	public function eliminar($id)
	{
		$this->paginas->eliminar_pagina($id);
		$this->session->set_flashdata('message', 'Página eliminada satisfactoriamente');

		redirect('admin/paginas', 'refresh');
	}


}
?>
