<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Posts_model extends Model
{
	// Variable protegida
	protected $_table;
	
	// Constructor
	public function __construct()
	{
		parent::Model();
		
		$this->_table = $this->config->item('database_tables');
	}

	// Métodos públicos
	public function listar_posts()
	{
		$this->db->select('posts.id, posts.titulo, posts.url_titulo, posts.fecha_posteo, posts.estado');
		$this->db->order_by('id', 'DESC');
			
		$query = $this->db->get($this->_table['posts'] . ' posts');
			
		if ($query->num_rows() > 0)
		{
			$result = $query->result_array();

			foreach (array_keys($result) as $key)
			{
				$result[$key]['cuenta_comentarios'] = $this->db->where('post_id', $result[$key]['id'])->from($this->_table['comentarios'])->count_all_results();
			}

			return $result;
		}
	}

	public function listar_post($id)
	{
		$this->db->select('id, fecha_posteo, titulo, excerpt, contenido, permitir_comentarios, sticky, estado');
		$this->db->where('id', $id);
			
		$query = $this->db->get($this->_table['posts'], 1);
			
		if ($query->num_rows() == 1)
		{
			return $query->row_array();
		}
	}
	

	public function crear_post()
	{
		$fecha_posteo = ($this->input->post('fecha_publicacion') != '') ? date('Y-m-d', strtotime($this->input->post('fecha_publicacion'))) : date('Y-m-d');
		$permitir_comentarios = ($this->input->post('permitir_comentarios') == 1) ? '1' : '0';
		$sticky = ($this->input->post('sticky') == 1) ? '1' : '0';
			
		$data = array
					(
						'autor' => $this->session->userdata('user_id'),
						'fecha_posteo' => $fecha_posteo,
						'titulo' => $this->input->post('titulo'),
						'url_titulo' => url_title($this->input->post('titulo'), 'dash', TRUE),
						'excerpt' => $this->input->post('excerpt'),
						'contenido' => $this->input->post('contenido'),
						'permitir_comentarios' => $permitir_comentarios,
						'sticky' => $sticky,
						'estado' => $this->input->post('estado')
					);
			
		$this->db->insert($this->_table['posts'], $data);
	}
	
	public function editar_post()
	{
		$fecha_posteo = ($this->input->post('fecha_publicacion') != '') ? date('Y-m-d', strtotime($this->input->post('fecha_publicacion'))) : date('Y-m-d');
		$permitir_comentarios = ($this->input->post('permitir_comentarios') == 1) ? '1' : '0';
		$sticky = ($this->input->post('sticky') == 1) ? '1' : '0';
			
		$data = array
					(
						'autor' => $this->session->userdata('user_id'),
						'fecha_posteo' => $fecha_posteo,
						'titulo' => $this->input->post('titulo'),
						'url_titulo' => url_title($this->input->post('titulo'), 'dash', TRUE),
						'excerpt' => $this->input->post('excerpt'),
						'contenido' => $this->input->post('contenido'),
						'permitir_comentarios' => $permitir_comentarios,
						'sticky' => $sticky,
						'estado' => $this->input->post('estado')
					);
			
		$this->db->where('id', $this->input->post('id'));
		$this->db->update($this->_table['posts'], $data);
	}

	public function eliminar_post($post_id)
	{
		$this->db->where('id', $post_id);
		$this->db->delete($this->_table['posts']);
		
		$this->db->where('post_id', $post_id);
		
		$this->comprobar_comentarios_abandonados();
	}
	
	// Métodos protegidos
	protected function comprobar_comentarios_abandonados()
	{
		$this->db->select('comentarios.id');
		$this->db->from($this->_table['comentarios'] . ' comentarios');
		$this->db->join($this->_table['posts'] . ' posts', 'comentarios.post_id = posts.id', 'left');
		$this->db->where($this->_table['posts'] . '.id IS NULL');
		
		$query = $this->db->get();
		
		// Si encontramos comentarios abandonados, los eliminamos
		if ($query->num_rows() > 0)
		{
			$results = $query->result_array();
			
			foreach ($results as $result)
			{
				$this->db->where('id', $result['id']);
				$this->db->delete($this->_table['comentarios']);
			}
		}
	}
	
}
?>
