<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Noticias extends Controller
{
	// Variable protegida
	protected $_template;
	
	// Constructor
	public function __construct()
	{
		parent::Controller();
			
		$this->template['module'] = 'noticias';

		// Cargamos modulos y librerias necesarias
		$this->load->module_model('noticias', 'noticias_model', 'noticias');
		$this->load->module_model('noticias', 'users_model', 'users');
		
		$this->load->library('user_agent');	
	}

	// Métodos públicos
	public function index($pagina = NULL)
	{
		$this->load->library('pagination');

		$config['total'] = $this->noticias->contar_posts();
		$config['por_pagina'] = $this->noticias->posts_por_pagina();
			
		$this->pagination->initialize($config);
			
		$contar_paginas = ceil($config['total'] / $config['por_pagina']);
		$pagina = ($pagina == 0) ? 1 : $pagina;
		$offset = $config['por_pagina'] * ($pagina - 1);
		
		if ($data['posts'] = $this->noticias->listar_posts($config['por_pagina'], $offset))
		{
			if ($pagina > $contar_paginas)
			{
				redirect('noticias', 'refresh');
			}
		
			$data['posts_por_pagina'] = $this->noticias->posts_por_pagina();
			$data['contar_posts'] = $this->noticias->contar_posts();
			$data['contar_paginas'] = $contar_paginas;
			$data['pagina_actual'] = $pagina;
			$data['siguiente_pagina'] = $pagina + 1;
			$data['pagina_anterior'] = $pagina - 1;

			foreach ($data['posts'] as $key => $post)
			{
				$data['posts'][$key]['url'] = post_url($post['url_titulo'], $post['fecha_posteo']);
				$data['posts'][$key]['display_name'] = $this->users->conseguir_display_name_usuario($post['autor']);
			}

			$this->_template['pagina']	= 'noticias/posts';
		}
		else
		{
			$this->_template['pagina']	= 'errors/no_posts';
		}
			
		$this->system_library->load($this->_template['pagina'], $data);
	}

	public function post($anno = NULL, $mes = NULL, $dia = NULL, $url_titulo = NULL)
	{
		$this->load->module_model('noticias', 'comentarios_model', 'comentarios');
		$this->load->module_model('noticias', 'users_model', 'users');
			
		if ($data['post'] = $this->noticias->conseguir_url_post($anno, $mes, $dia, $url_titulo))
		{
			$data['post']['url'] = post_url($data['post']['url_titulo'], $data['post']['fecha_posteo']);
			$data['post']['display_name'] = $this->users->conseguir_display_name_usuario($data['post']['autor']);
			
			if ($data['post']['permitir_comentarios'] == 1)
			{
				$this->comentario($data['post']['id'], $data['post']['url']);
			}
				
			$data['comentarios'] = $this->comentarios->listar_comentarios($data['post']['id']);

			if ($data['comentarios'] != "")
			{
				foreach ($data['comentarios'] as $key => $comentario)
				{
					$data['comentarios'][$key]['contenido']  = $comentario['contenido'];

					if ($comentario['user_id'] != "")
					{
						$website = $this->users->conseguir_website_usuario($comentario['user_id']);
						$display_name = $this->users->conseguir_display_name_usuario($comentario['user_id']);
						$data['comentarios'][$key]['autor'] = '<a href="' . prep_url($website) . '" target="_blank">' . $display_name . '</a>';
					}
					else
					{
						if ($comentario['autor_website'] != "")
						{
							$data['comentarios'][$key]['autor'] = '<a href="' . prep_url($comentario['website_autor']) . '" target="_blank">' . $comentario['autor'] . '</a>';
						}
					}
				}
			}

			$this->_template['pagina']	= 'noticias/single_post';
		}
		else
		{
			$this->_template['pagina']	= 'errors/404';
		}
			
		$this->system_library->load($this->_template['pagina'], $data);
	}


	public function comentario($id, $url)
	{
		$this->load->module_model('noticias', 'comentarios_model', 'comentarios');
			
		if ($this->session->userdata('logged_in') == FALSE)
		{
			$this->form_validation->set_rules('nickname', 'Nick', 'required|max_length[50]|xss_clean');
			$this->form_validation->set_rules('email', 'Email', 'required|valid_email');
		}
		
		$this->form_validation->set_rules('website', 'Website', 'xss_clean');
		$this->form_validation->set_rules('comentario', 'Comentario', 'required|max_length[400]|htmlentities');
			
		$this->form_validation->set_error_delimiters('', '<br />');

		if ($this->form_validation->run() == TRUE)
		{
			$this->comentarios->crear_comentario($id);
			redirect($url, 'refresh');
		}
	}
}
?>
