<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Noticias_model extends Model
{
	// Variable protegida
	protected $_table;
	
	// Constructor
	public function __construct()
	{
		parent::Model();

		$this->_table = $this->config->item('database_tables');
	}

	// Métodos públicos
	public function posts_por_pagina()
	{
		$this->db->select('valor');
		$this->db->where('nombre', 'posts_por_pagina');
			
		$query = $this->db->get($this->_table['settings'], 1);
			
		if ($query->num_rows() == 1)
		{
			$row = $query->row_array();
		}
			
		return $row['valor'];
	}

	public function contar_posts()
	{
		$this->db->select('id');
		$this->db->where('estado', 'publicado');
			
		$query = $this->db->count_all_results($this->_table['posts']);
			
		return $query;
	}

	public function listar_posts($numero = 10, $offset = 0)
	{
		$fecha_actual = date('Y-m-d');
		
		$this->db->select('posts.id, posts.autor, posts.fecha_posteo, posts.titulo, posts.url_titulo, posts.excerpt, posts.contenido, posts.permitir_comentarios, posts.sticky, posts.estado, posts.autor, usuarios.display_name');
		$this->db->from($this->_table['posts']);// . ' posts');
		$this->db->join($this->_table['usuarios'],'posts.autor = usuarios.id');// . ' usuarios', 'posts.autor = usuarios.id');
		$this->db->where('posts.estado', 'publicado');
		$this->db->where('posts.fecha_posteo <=', $fecha_actual);
		$this->db->order_by('sticky', 'DESC');
		$this->db->order_by('id', 'DESC');
		$this->db->limit($numero, $offset);
			
		$query = $this->db->get();
			
		if ($query->num_rows() > 0)
		{
			$result = $query->result_array();
			
			foreach (array_keys($result) as $key)
			{
				$result[$key]['cuenta_comentarios'] = $this->db->where('post_id', $result[$key]['id'])->from($this->_table['comentarios'])->count_all_results();
			}

			return $result;
		}
	}

	public function listar_posts_por_fecha($anno, $mes)
	{
		$fecha = $anno . '-' . $mes;
		$fecha_actual = date('Y-m-d');
		
		$this->db->select('posts.id, posts.autor, posts.fecha_posteo, posts.titulo, posts.url_titulo, posts.excerpt, posts.contenido, posts.permitir_comentarios, posts.sticky, posts.estado, posts.autor, usuarios.display_name');
		$this->db->from($this->_table['posts'] . ' posts');
		$this->db->join($this->_table['usuarios'] . ' usuarios', 'posts.autor = usuarios.id');
		$this->db->where('posts.estado', 'publicado');
		$this->db->where('posts.fecha_posteo <=', $fecha_actual);
		$this->db->like('posts.fecha_posteo', $fecha);
		$this->db->order_by('posts.sticky', 'DESC');
		$this->db->order_by('posts.id', 'DESC');
			
		$query = $this->db->get();
			
		if ($query->num_rows() > 0)
		{
			$result = $query->result_array();
			
			foreach (array_keys($result) as $key)
			{
				$result[$key]['cuenta_comentarios'] = $this->db->where('post_id', $result[$key]['id'])->from($this->_table['comentarios'])->count_all_results();
			}

			return $result;
		}
	}

	public function conseguir_url_post($anno, $mes, $dia, $url_titulo)
	{
		$fecha = $anno . '-' . $mes . '-' . $dia;
		
		$this->db->select('posts.id, posts.autor, posts.fecha_posteo, posts.titulo, posts.url_titulo, posts.excerpt, posts.contenido, posts.permitir_comentarios, posts.sticky, posts.estado, posts.autor, usuarios.display_name');
		$this->db->from($this->_table['posts']);// . ' posts');
		$this->db->join($this->_table['usuarios'], 'posts.autor = usuarios.id');// . ' usuarios', 'posts.autor = usuarios.id');
		$this->db->where('posts.estado', 'publicado');
		$this->db->where('posts.url_titulo', $url_titulo);
		$this->db->where('posts.fecha_posteo', $fecha);
		$this->db->limit(1);
			
		$query = $this->db->get();
			
		if ($query->num_rows() == 1)
		{
			$result = $query->row_array();
			
			$result['cuenta_comentarios'] = $this->db->where('post_id', $result['id'])->from($this->_table['comentarios'])->count_all_results();

			return $result;
		}
	}
	
	public function conseguir_id_post($post_id)
	{
		$this->db->select('posts.id, posts.autor, posts.fecha_posteo, posts.titulo, posts.url_titulo, posts.excerpt, posts.contenido, posts.permitir_comentarios, posts.sticky, posts.estado, posts.autor, usuarios.display_name');
		$this->db->from($this->_table['posts']);// . ' posts');
		$this->db->join($this->_table['usuarios'],'posts.autor = usuarios.id');//. ' usuarios', 'posts.autor = usuarios.id');
		$this->db->where('posts.estado', 'publicado');
		$this->db->where('posts.id', $post_id);
		$this->db->limit(1);
		
		$query = $this->db->get();
			
		if ($query->num_rows() > 0)
		{
			$result = $query->result_array();
			
			foreach (array_keys($result) as $key)
			{
				$result['cuenta_comentarios'] = $this->db->where('post_id', $result['id'])->from($this->_table['comentarios'])->count_all_results();
			}

			return $result;
		}
	}
}
?>
