<!--###############################################################################
## OpenConference project
##
## Copyright (C) 2009 Antonio Falcón Aragón  antonio.falconaragon@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################-->

<?php

class Usuario extends Controller
{
	// Variable protegida
	protected $_template;
	
	// Constructor
	public function __construct()
	{
		parent::Controller();

		// Cargamos modulos y librerias necesarias
		$this->load->module_model('usuario', 'usuario_model', 'usuario');
		
		$this->load->library('phpass_library');
	}

	// Métodos públicos
	public function registro()
	{
		if (!$this->access_library->is_logged_in())
		{
			if ($this->system_library->settings['permitir_registros'] == 1)
			{
				$this->form_validation->set_rules('username', 'Username', 'required|max_length[50]|callback_comprueba_username|xss_clean');
				$this->form_validation->set_rules('display_name', 'Display Name', 'max_length[50]|xss_clean');
				$this->form_validation->set_rules('password', 'Password', 'required|matches[password_retype]');
				$this->form_validation->set_rules('password_retype', 'Confirmacion Password', 'required');
				$this->form_validation->set_rules('email', 'Email', 'required|valid_email|callback_comprueba_email');
				$this->form_validation->set_rules('website', 'Website', 'xss_clean');;			

				$this->form_validation->set_error_delimiters('', '<br />');
					
				if ($this->form_validation->run() == TRUE)
				{
					$this->usuario->crear_usuario();
					$this->session->set_flashdata('message', 'Usuario creado satisfactoriamente');
		
					redirect('usuario/registro', 'refresh');
				}
			}
					
			$this->_template['pagina']	= 'usuario/registration';
					
			$this->system_library->load($this->_template['pagina']);
		}
		else
		{
			redirect('noticias', 'refresh');
		}
	}

	public function activacion_cuenta()
	{
		$key = $this->uri->segment(4);
		$email = $this->uri->segment(6);

		if ($key != '' && $email != '')
		{
			if ($this->usuario->check_secret_key($key, $email))
			{
				$this->usuario->activa_cuenta($key, $email);
				
				$this->session->set_flashdata('message', 'Cuenta activada satisfactoriamente');
				
				redirect('usuario/registro', 'refresh');
			}
			else 
			{
				$this->_template['pagina']	= 'errors/invalid_secret_key';
				
				$this->system_library->load($this->_template['pagina']);
			}
		}
		else
		{
			redirect('usuario/login', 'refresh');
		}
	}
	
	public function password_olvidado()
	{	
		$key = $this->uri->segment(4);
		$email = $this->uri->segment(6);
		
		if ($key == null && $email == null)
		{
			$this->form_validation->set_rules('username', 'Username', 'required|callback_existe_usuario');
			$this->form_validation->set_rules('email', 'Email', 'required|valid_email|callback_existe_email');
						
			$this->form_validation->set_error_delimiters('', '<br />');
			
			if ($this->form_validation->run() == TRUE)
			{
				$this->usuario->password_olvidado();
				
				$this->session->set_flashdata('message', 'Se ha enviado correctamente el email');
				
				redirect('usuario/password_olvidado', 'refresh');	
			}
			
			$this->_template['pagina']	= 'usuario/forgotten_password';
			
			$this->system_library->load($this->_template['pagina']);
		}
		else
		{
			if ($this->user->check_secret_key($key, $email))
			{
				$this->user->reestablecer_password($key, $email);

				$this->session->set_flashdata('message', 'Password reestablecido satisfactoriamente');
				
				redirect('usuario/password_olvidado', 'refresh');
			}
			else 
			{
				$this->_template['pagina']	= 'errors/invalid_secret_key';
				
				$this->system_library->load($this->_template['pagina']);
			}
		}
	}
	
	public function ver($nickname = NULL)
	{
		
		$this->access_library->check_logged_in();
		
		if ($data['usuario'] = $this->usuario->conseguir_usuario_por_nick($nickname))
		{
			$this->_template['pagina']	= 'usuario/view';
		}
		else
		{
			$this->_template['pagina']	= 'errors/404';
		}
				
		$this->system_library->load($this->_template['pagina'], $data);
	}
	
	public function perfil()
	{	
		$this->access_library->check_logged_in();
		
		$id = $this->session->userdata('user_id');

		$this->form_validation->set_rules('display_name', 'Display Name', 'max_length[50]|xss_clean');
		$this->form_validation->set_rules('password', 'Password', 'matches[password_retype]');
		$this->form_validation->set_rules('password_retype', 'Confirmacion Password', '');
		$this->form_validation->set_rules('email', 'Email', 'required|valid_email');
		$this->form_validation->set_rules('website', 'Website', 'xss_clean');;	
				
		$this->form_validation->set_error_delimiters('', '<br />');
				
		$data['usuario'] = $this->usuario->listar_usuario($id);

		if ($this->form_validation->run() == TRUE)
		{
			$this->usuario->editar_perfil($id);
				
			if ($this->input->post('password') != "")
			{
				$this->usuario->logout();
				redirect('usuario/login', 'refresh');
			}
				
			$this->session->set_flashdata('message', 'Perfil editado satisfactoriamente');
	
			redirect('usuario/perfil', 'refresh');
		}
				
		$this->_template['pagina']	= 'usuario/profile';
	
		$this->system_library->load($this->_template['pagina'], $data);
	}
	

	public function login()
	{	
		if (!$this->access_library->is_logged_in())
		{
			if ($this->input->post('username') != "" && $this->input->post('password') != "")
			{
				$this->usuario->verificar_usuario($this->input->post('username'), $this->input->post('password'));
					
				if ($this->session->userdata('nivel') == 'usuario' || $this->session->userdata('nivel') == 'administrador' || $this->session->userdata('nivel') == 'revisor')
				{
					$this->usuario->actualizar_ultimo_acceso();
					redirect('noticias', 'refresh');
				}
				else
				{
					redirect('usuario/login', 'refresh');
				}
			}
			else
			{
				$this->_template['pagina']	= 'usuario/login';
					
				$this->system_library->load($this->_template['pagina']);
			}
		}
		else
		{
			redirect('noticias', 'refresh');
		}
	}

	public function logout()
	{
		if ($this->access_library->is_logged_in())
		{
			$this->usuario->logout();

			redirect('noticias', 'refresh');
		}
		else
		{
			redirect('noticias', 'refresh');
		}
	}
	
	// Funciones de validación personalizadas
	public function comprueba_username($username)
	{
		if ($this->usuario->comprobar_validacion(array('username' => $username)))
		{
			$this->form_validation->set_message('comprobacion_username', 'El username escogido ya existe');
			
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	
	public function comprueba_email($email)
	{
		if ($this->usuario->comprobar_validacion(array('email' => $email)))
		{
			$this->form_validation->set_message('comprobacion_email', 'El email aportado ya existe');
			
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	
	public function existe_usuario($username)
	{
		if (!$this->usuario->comprobar_validacion(array('username' => $username)))
		{
			$this->form_validation->set_message('existe_usuario', 'El usuario ya existe');
			
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
	
	public function existe_email($email)
	{
		if (!$this->usuario->comprobar_validacion(array('email' => $email)))
		{
			$this->form_validation->set_message('existe_email', 'El email ya existe');
			
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
}
?>
